#! /bin/sh

VSN=1.2.0

if (test -z "$1" -o $# -ne 1) ; then
    echo " Usage: setup.sh OTP_DIR"
    echo " where: OTP_DIR   -- full path name of the Erlang/OTP"
    echo "                     installation directory"
    exit
fi

DIALYZER_OTP_DIR=$1

echo "Verifying Erlang/OTP installation..."
if (test -f $DIALYZER_OTP_DIR/lib/erlang/bin/erl) ; then
    # The user pointed out the top-level of an Erlang/OTP
    # installed using 'make install'. Silently descend to
    # the correct root directory.
    DIALYZER_OTP_DIR="$DIALYZER_OTP_DIR/lib/erlang"
fi

DIALYZER_OTP_OTP="$DIALYZER_OTP_DIR/bin/erl"
DIALYZER_OTP_ERLC="$DIALYZER_OTP_DIR/bin/erlc"

if (test -f "$DIALYZER_OTP_OTP") ; then
    echo "  $DIALYZER_OTP_OTP exists..."
else
    echo "  Missing $DIALYZER_OTP_OTP -- check your Erlang/OTP installation"
    exit 1;
fi

if (test -f "$DIALYZER_OTP_ERLC") ; then
    echo "  $DIALYZER_OTP_ERLC exists..."
else
    echo "  Missing $DIALYZER_OTP_ERLC -- check your Erlang/OTP installation"
    exit 1;
fi

echo "Checking dialyzer directory..."
if (test -d ebin) ; then
    echo "  The 'ebin' directory already exists..."
else
    echo "  Creating 'ebin/'"
    mkdir ebin
fi

if (test -d plt) ; then
    echo "  The 'plt'  directory already exists..."
else
    echo "  Creating 'plt/'"
    mkdir plt
fi

if (test -f plt/dialyzer_empty_plt) ; then
    echo "  An empty persistent lookup table (PLT) already exists..."
else
    echo "  Creating empty persistent lookup table (PLT)"
    touch plt/dialyzer_empty_plt
fi

if (test -x "$DIALYZER_OTP_ERLC") ; then
    ## remove the file unconditionally to avoid problems when using
    ## symbolic links across filesystems in different machines.
    rm -f src/hipe_icode_type.hrl
    echo "  Creating a symbolic link to $DIALYZER_OTP_DIR/hipe_icode_type.hrl"
    ln -s $DIALYZER_OTP_DIR/lib/hipe*/icode/hipe_icode_type.hrl src
    ## create all files in src
    cd src
    sed -e "s;%VSN%;$VSN;" dialyzer.hrl.src > dialyzer.hrl
    for file in *.erl ; do
      echo "    Compiling $file"
      $DIALYZER_OTP_ERLC -W +debug_info -o../ebin $file
      beam_file=`basename $file .erl`".beam"
      if (test ! -f ../ebin/$beam_file); then
	  echo "Compilation of $file failed"
	  exit 1
      fi
    done
    sed -e "s;%VSN%;$VSN;" dialyzer.app.src > ../ebin/dialyzer.app
    cd ..

    ## check the .beam files for consistency
    file="utils/dialyzer_chk.erl"
    $DIALYZER_OTP_ERLC -W -o utils $file
    beam_file=`basename $file .erl`".beam"
    if (test ! -f utils/$beam_file); then
       echo "Compilation of $file failed"
       exit 1
    fi
    printf "  Checking consistency of ebin/*.beam files... "
    $DIALYZER_OTP_OTP -pa utils/ -noshell -s dialyzer_chk test `pwd` -s init stop
    rm -f utils/dialyzer_chk.beam

    ## create the dialyzer script
    sed -e "s;%OTP_DIR%;$DIALYZER_OTP_DIR;" \
	-e "s;%VSN%;$VSN;" -e "s;%DATE%;`date`;" \
	-e "s;%DIALYZER_DIR%;`pwd`;" src/dialyzer.src > dialyzer
    chmod +x dialyzer
    echo "Dialyzer is analyzing itself..."
    ./dialyzer --src --command-line "`pwd`/src"
    echo "Dialyzer setup done!"
    echo "  For the full set of Dialyzer options type: ./dialyzer --help"
else
    echo "Could not find ${DIALYZER_OTP_ERLC}"
fi
